<?php declare (strict_types=1);

namespace EuKit\Base\Parts;

use Infomaker\Imengine\Imengine;
use SimpleXMLIterator;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\Base\Models\Post;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\NewsML\Parsers\RelatedLinksParser;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;

class Concept extends GenericPropertyObject
{
    public function __construct(PropertyObject $object)
    {
        $this->fill($object->getProperties());
    }

    public function getAvatar(): ?SimpleImage
    {
        $uuid = $this->getMultiValue('image_uuids');

        if (empty($uuid)) {
            return null;
        }

        $width = 115;
        $height = 115;

        return new SimpleImage([
            'src' => Imengine::thumbnail($width, $height)->with(['format' => 'png'])->fromUuid($uuid[0]),
            'width' => $width,
            'height' => $height,
        ]);
    }

    public function getMetadata(): array
    {
        $newsml = $this->getSingleValue('meta_data');

        if (empty($newsml)) {
            return [];
        }

        $transformer = NewsMLTransformerManager::createMetaDataTransformer();

        $result = $transformer->transform($newsml);

        if (empty($result)) {
            return [];
        }

        $data = array_shift($result);

        if ($data instanceof Item) {
            return $data->getProperties();
        }
    }

    public function getRelatedLinks(): array
    {
        libxml_use_internal_errors(true);
        $newsml = $this->getSingleValue('related_links_raw');

        if (empty($newsml)) {
            return [];
        }

        $result = [];

        try {
            $relatedLinkParser = new RelatedLinksParser();

            return $relatedLinkParser->parseLinks(new SimpleXMLIterator($newsml));
        } catch (Exception $exception) {
            NewRelicLog::error(sprintf('Failed parsing Concept NewsML from %s', $newsml), $exception);
        }

        return [];
    }

    /**
     * Returns new Concept based on uuid from post.
     *
     * @param Post $post
     * @param string $propertyMap
     * @return Concept
     * @throws \UnexpectedValueException
     * @since 0.4
     */
    public static function createFromPost(Post $post, $propertyMap = 'Concept')
    {
        $uuid = $post->getMeta('oc_uuid');

        $provider = OpenContentProvider::setup();
        $provider->setPropertyMap($propertyMap);
        $object = $provider->getObject($uuid);

        if($object === null){
            throw new \UnexpectedValueException('Unable to fetch Concept from OC using uuid from post.');
        }

        return new static($object);
    }
}
